@@ -1,10 +1,7 @@
-__version__ = '0.12.9'
+__version__ = '0.13.0'
 
 
-from .collection import *
 from .core import *
-from .func import *
-
 
 # We add utilities for convenience to the common namespace:
 from .core.utils import _info_appr
@@ -13,3 +10,11 @@ from .core.utils import _maxvol
 from .core.utils import _ones
 from .core.utils import _range
 from .core.utils import _reshape
+from .core.utils import _vector_index_expand
+from .core.utils import _vector_index_prepare
+
+
+from .func import *
+
+
+#import core_jax
