@@ -9,7 +9,7 @@ from .act_many import outer_many
 
 
 from .act_one import copy
-from .act_one import interface_matrices
+from .act_one import interface
 from .act_one import get
 from .act_one import get_and_grad
 from .act_one import get_many
@@ -77,8 +77,9 @@ from .grid import ind_to_poi
 from .grid import ind_tt_to_qtt
 from .grid import poi_scale
 from .grid import poi_to_ind
-from .grid import sample_lhs
-from .grid import sample_tt
+
+
+from .matrices import matrix_delta
 
 
 from .maxvol import maxvol
@@ -91,15 +92,23 @@ from .optima import optima_tt_beam
 from .optima import optima_tt_max
 
 
+from .optima_contin import optima_contin_tt_beam
+
+
 from .props import erank
 from .props import ranks
 from .props import shape
 from .props import size
 
 
+from .sample import sample
+from .sample import sample_square
+from .sample import sample_lhs
+from .sample import sample_tt
+
+
 from .stat import cdf_confidence
 from .stat import cdf_getter
-from .stat import sample_ind_rand
 
 
 from .svd import matrix_skeleton
@@ -109,6 +118,14 @@ from .svd import svd_matrix
 from .svd import svd_incomplete
 
 
+from .tensors import const
+from .tensors import delta
+from .tensors import poly
+from .tensors import rand
+from .tensors import rand_custom
+from .tensors import rand_norm
+
+
 from .transformation import full
 from .transformation import full_matrix
 from .transformation import orthogonalize
@@ -117,4 +134,7 @@ from .transformation import orthogonalize_right
 from .transformation import truncate
 
 
+from .vectors import vector_delta
+
+
 from .vis import show
