@@ -1,8 +1,12 @@
 """Package teneva, module func.func: class that represents the function."""
-from mpl_toolkits.mplot3d import Axes3D
-from matplotlib import cm
-import matplotlib.pyplot as plt
-from matplotlib.ticker import LinearLocator, FormatStrFormatter
+try:
+    from mpl_toolkits.mplot3d import Axes3D
+    from matplotlib import cm
+    import matplotlib.pyplot as plt
+    from matplotlib.ticker import LinearLocator, FormatStrFormatter
+    WITH_MATPLOTLIB = True
+except Exception as e:
+    WITH_MATPLOTLIB = False
 import numpy as np
 from time import perf_counter as tpc
 
@@ -12,6 +16,7 @@ from .utils import _data_load
 from .utils import _data_prep
 from .utils import _data_save
 from .utils import _noise
+import teneva
 from teneva import als
 from teneva import anova
 from teneva import cache_to_data
@@ -25,7 +30,6 @@ from teneva import grid_flat
 from teneva import ind_to_poi
 from teneva import sample_lhs
 from teneva import size
-from teneva import tensor_rand
 from teneva import truncate
 
 
@@ -816,6 +820,9 @@ class Func:
             k (int): number of points for each dimension.
 
         """
+        if not WITH_MATPLOTLIB:
+            raise ValueError('Matplotlib is not installed')
+
         if self.d != 2:
             raise ValueError('Plot is supported only for 2D case')
 
@@ -877,7 +884,7 @@ class Func:
         self.method = ''
 
         t = tpc()
-        Y = tensor_rand(self.n, r)
+        Y = teneva.rand_custom(self.n, r)
         self.t += tpc() - t
 
         self.prep(Y, with_int=False)
